{% extends "email_layout.html" %}

{% block content %}
<p>{{place}}:Kill/Takedown Notice Kill/Takedown Notice</p>
<p>**KILL/TAKEDOWN NOTICE**  **KILL/TAKEDOWN NOTICE**</p>

    {{body | safe}}

{% endblock %}
